@@ -45,6 +45,7 @@ from conda.exceptions import (
 from conda.gateways.disk.create import mkdir_p
 from conda.gateways.disk.delete import delete_trash, path_is_clean, rm_rf
 from conda.gateways.disk.test import is_conda_environment
+from conda.lock import FileLock
 from conda.misc import explicit, touch_nonadmin
 from conda.models.match_spec import MatchSpec
 
@@ -140,13 +141,14 @@ def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
         raise DryRunExit()
 
     try:
-        unlink_link_transaction.download_and_extract()
-        if context.download_only:
-            raise CondaExitZero(
-                "Package caches prepared. UnlinkLinkTransaction cancelled with "
-                "--download-only option."
-            )
-        unlink_link_transaction.execute()
+        with FileLock(PackageCacheData.first_writable().pkgs_dir):
+            unlink_link_transaction.download_and_extract()
+            if context.download_only:
+                raise CondaExitZero(
+                    "Package caches prepared. UnlinkLinkTransaction cancelled with "
+                    "--download-only option."
+                )
+            unlink_link_transaction.execute()
 
     except SystemExit as e:
         raise CondaSystemExit("Exiting", e)
@@ -258,11 +260,12 @@ def remove(args, parser):
             exit_code = 1
             return exit_code
 
-        package_cache = api.MultiPackageCache(context.pkgs_dirs)
-        transaction = api.Transaction(
-            solver, package_cache, PackageCacheData.first_writable().pkgs_dir
-        )
-        downloaded = transaction.prompt(repos)
+        with FileLock(PackageCacheData.first_writable().pkgs_dir):
+            package_cache = api.MultiPackageCache(context.pkgs_dirs)
+            transaction = api.Transaction(
+                solver, package_cache, PackageCacheData.first_writable().pkgs_dir
+            )
+            downloaded = transaction.prompt(repos)
         if not downloaded:
             exit(0)
 
@@ -404,7 +407,7 @@ def install(args, parser, command="install"):
             )
             return
 
-        specs.extend([MatchSpec(s) for s in file_specs])
+        specs.extend(MatchSpec(s) for s in file_specs)
 
     specs.extend(specs_from_args(args_packages, json=context.json))
 
@@ -432,8 +435,9 @@ def install(args, parser, command="install"):
     # and that they are name-only specs
     if isupdate and context.update_modifier == UpdateModifier.UPDATE_ALL:
         for i in installed_names:
-            if i != "python":
-                specs.append(MatchSpec(i))
+            if i.startswith("__"):
+                continue
+            specs.append(MatchSpec(i))
 
         prefix_data = PrefixData(prefix)
         for s in args_packages:
@@ -458,7 +462,7 @@ def install(args, parser, command="install"):
                 ms = MatchSpec(dep)
                 if ms.name != "python":
                     final_specs.append(MatchSpec(ms.name))
-        specs = set(final_specs)
+        specs = list(set(final_specs))
 
     if newenv and args.clone:
         if args.packages:
@@ -483,17 +487,17 @@ def install(args, parser, command="install"):
     if not (context.quiet or context.json):
         print("\nLooking for: {}\n".format([str(s) for s in specs]))
 
-    spec_names = [s.name for s in specs]
-
     # If python was not specified, check if it is installed.
     # If yes, add the installed python to the specs to prevent updating it.
     python_constraint = None
 
-    if "python" not in spec_names:
-        if "python" in installed_names:
-            i = installed_names.index("python")
-            version = installed_pkg_recs[i].version
-            python_constraint = MatchSpec("python==" + version).conda_build_form()
+    if "python" in installed_names:
+        if context.update_modifier == UpdateModifier.UPDATE_ALL or not any(
+            s.name == "python" for s in specs
+        ):
+            version = installed_pkg_recs[installed_names.index("python")].version
+            major_minor_version = ".".join(version.split(".")[:2])
+            python_constraint = f"python {major_minor_version}.*"
 
     mamba_solve_specs = [s.__str__() for s in specs]
 
@@ -504,8 +508,9 @@ def install(args, parser, command="install"):
 
     repos = []
 
-    prefix_data = api.PrefixData(context.target_prefix)
-    prefix_data.load()
+    with FileLock(context.target_prefix):
+        prefix_data = api.PrefixData(context.target_prefix)
+        prefix_data.load()
 
     # add installed
     if use_mamba_experimental:
@@ -555,8 +560,8 @@ def install(args, parser, command="install"):
 
         pinned_specs_info = ""
         if python_constraint:
-            solver.add_pin(python_constraint)
             pinned_specs_info += f"  - {python_constraint}\n"
+            solver.add_pin(python_constraint)
 
         pinned_specs = get_pinned_specs(context.target_prefix)
         if pinned_specs:
@@ -594,28 +599,29 @@ def install(args, parser, command="install"):
             exit_code = 1
             return exit_code
 
-        package_cache = api.MultiPackageCache(context.pkgs_dirs)
-        transaction = api.Transaction(
-            solver, package_cache, PackageCacheData.first_writable().pkgs_dir
-        )
-        mmb_specs, to_link, to_unlink = transaction.to_conda()
+        with FileLock(PackageCacheData.first_writable().pkgs_dir):
+            package_cache = api.MultiPackageCache(context.pkgs_dirs)
+            transaction = api.Transaction(
+                solver, package_cache, PackageCacheData.first_writable().pkgs_dir
+            )
+            mmb_specs, to_link, to_unlink = transaction.to_conda()
 
-        specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
-        specs_to_remove = [MatchSpec(m) for m in mmb_specs[1]]
+            specs_to_add = [MatchSpec(m) for m in mmb_specs[0]]
+            specs_to_remove = [MatchSpec(m) for m in mmb_specs[1]]
 
-        transaction.log_json()
+            transaction.log_json()
+            downloaded = transaction.prompt(repos)
+            if not downloaded:
+                exit(0)
 
-        downloaded = transaction.prompt(repos)
-        if not downloaded:
-            exit(0)
-        PackageCacheData.first_writable().reload()
+            PackageCacheData.first_writable().reload()
 
     # if use_mamba_experimental and not os.name == "nt":
     if use_mamba_experimental:
         if newenv and not isdir(context.target_prefix) and not context.dry_run:
             mkdir_p(prefix)
-
-        transaction.execute(prefix_data)
+        with FileLock(PackageCacheData.first_writable().pkgs_dir):
+            transaction.execute(prefix_data)
     else:
         conda_transaction = to_txn(
             specs_to_add,
@@ -857,6 +863,9 @@ def _wrapped_main(*args, **kwargs):
     elif "MAMBA_NO_BANNER" in os.environ:
         print_banner = False
 
+    if "list" in argv:
+        print_banner = False
+
     args = argv
 
     p = generate_parser()
@@ -888,7 +897,8 @@ def main(*args, **kwargs):
     init_std_stream_encoding()
 
     if "activate" in sys.argv or "deactivate" in sys.argv:
-        print("Use conda to activate / deactivate the environment.")
+        print("Run 'mamba init' to be able to run mamba activate/deactivate")
+        print("and start a new shell session. Or use conda to activate/deactivate.")
         print("\n    $ conda " + " ".join(sys.argv[1:]) + "\n")
         return sys.exit(-1)
 
